@@ -37,7 +37,7 @@ sub new {
 ##  + %opts: clobber %$rel
 sub create {
   my ($rel,$coldb,$datfile,%opts) = @_;
-  $rel->logconfess("create(): abstract method called");
+  $rel->logconfess($coldb->{error}="create(): abstract method called");
 }
 
 ##==============================================================================
@@ -51,7 +51,7 @@ sub create {
 ##  + implicitly flushes the new index
 sub union {
   my ($rel,$coldb, $pairs,%opts) = @_;
-  $rel->logconfess("union(): abstract method called");
+  $rel->logconfess($coldb->{error}="union(): abstract method called");
 }
 
 ##==============================================================================
@@ -99,7 +99,7 @@ sub dbinfo {
 ##     strings => $bool,          ##-- do/don't stringify item keys (default=do)
 ##     packed  => $bool,          ##-- leave item keys packed (default=don't)
 ##     fill    => $bool,          ##-- if true, returned multi-profile will have null profiles inserted for missing slices
-##     onepass => $bool,          ##-- if true, use fast but incorrect 1-pass method (Cofreqs subclass only)
+##     onepass => $bool,          ##-- if true, use fast but incorrect 1-pass method (Cofreqs subclass only, >= v0.09.001)
 ##    )
 ##  + default implementation
 ##    - parses request and extracts target tuple-ids
@@ -155,9 +155,9 @@ sub profile {
     $nbits   //= $ac->{a2t}{len_i}*8;
     $pack_tv //= "$ac->{a2t}{pack_i}*";
     vec($test_tv='',0,$nbits) = 0x12345678 if (!defined($test_tv));
-    $reldb->logconfess("profile(): multimap pack-size mismatch: nbits($ac->{a2t}{base}.*) != $nbits")
+    $reldb->logconfess($coldb->{error}="profile(): multimap pack-size mismatch: nbits($ac->{a2t}{base}.*) != $nbits")
       if ($ac->{a2t}{len_i} != $nbits/8);
-    $reldb->logconfess("profile(): multimap pack-template '$ac->{a2t}{pack_i}' for $ac->{a2t}{base}.* is not big-endian")
+    $reldb->logconfess($coldb->{error}="profile(): multimap pack-template '$ac->{a2t}{pack_i}' for $ac->{a2t}{base}.* is not big-endian")
       if (pack($ac->{a2t}{pack_i},0x12345678) ne $test_tv);
 
     ##-- target set construction
@@ -175,7 +175,7 @@ sub profile {
   }
   my $tis = [$tivec ? unpack($pack_tv, $tivec) : qw()];
 
-  ##-- prepare: parse and filter tuples
+  ##-- parse date request (no filtering here)
   $reldb->vlog($logProfile, "profile(): parse date request (date=$opts{date}, slice=$opts{slice}, fill=$opts{fill})");
   my $dreq = $opts{dreq} = $coldb->parseDateRequest(@opts{qw(date slice fill)});
 
@@ -217,6 +217,54 @@ sub profile {
   return $mp;
 }
 
+##--------------------------------------------------------------
+## Relation API: extend (pass-2 for multi-clients)
+
+## $mprf = $rel->extend($coldb, %opts)
+##  + get independent f2 frequencies for \%slice2keys = $opts{slice2keys}
+##  + calls $rel->subextend(\%slice2keys, \%opts)
+##  + returns a DiaColloDB::Profile::Multi containing the appropriate f2 entries
+sub extend {
+  my ($reldb,$coldb,%opts) = @_;
+
+  ##-- common variables
+  $opts{coldb}   = $coldb; ##-- pass-down to subprofile() methods
+  my $logProfile = $coldb->{logProfile};
+
+  ##-- sanity check(s)
+   ##-- sanity check(s)
+  if (!$opts{slice2keys}) {
+     $reldb->logwarn($coldb->{error}="extend(): no 'slice2keys' parameter specified!");
+     return undef;
+   }
+  elsif (!UNIVERSAL::isa($opts{slice2keys},'HASH')) {
+     $reldb->logwarn($coldb->{error}="extend(): failed to parse 'slice2keys' parameter");
+     return undef;
+   }
+
+  ##-- subprofile2() requirements
+  my $groupby= $opts{groupby} = $coldb->groupby($opts{groupby});
+  my $a2data = $opts{a2data}  = {map {($_->{a}=>$_)} @{$coldb->attrData($coldb->attrs)}};
+  my $dreq   = $opts{dreq}    = $coldb->parseDateRequest(@opts{qw(date slice fill)});
+
+  ##-- create dummy profiles for completion
+  my $s2prf   = {};
+  my ($slice,$keys);
+  while (($slice,$keys) = each %{$opts{slice2keys}}) {
+    $s2prf->{$slice} = DiaColloDB::Profile->new(label=>$slice, N=>0,f1=>0,f12=>{ map {($_=>0)} @$keys })->stringify($groupby->{s2g});
+  }
+
+  ##-- get independent f2
+  $reldb->subextend($s2prf, \%opts);
+
+  ##-- collect, stringify & return
+  my $mp = DiaColloDB::Profile::Multi->new(profiles=>[@$s2prf{sort {$a<=>$b} keys %$s2prf}],
+					   titles  =>$groupby->{titles});
+  $reldb->vlog($logProfile, "extend(): stringify");
+  $mp->stringify($groupby->{g2s});
+
+  return $mp;
+}
 
 ##--------------------------------------------------------------
 ## Relation API: comparison (diff)
@@ -294,7 +342,10 @@ sub diff {
 
 
 ##==============================================================================
-## Relation API: default: subprofile1()
+## Relation API: default
+
+##--------------------------------------------------------------
+## Relation API: default: subprofile1
 
 ## \%slice2prf = $rel->subprofile1(\@tids,\%opts)
 ##  + get slice-wise joint frequency profile(s) for \@tids (db must be opened)
@@ -304,9 +355,12 @@ sub diff {
 ##     dreq  => \%dreq,    ##-- parsed date request
 sub subprofile1 {
   my ($rel,$tids,$opts) = @_;
-  $rel->logconfess("subprofile(): abstract method called");
+  $rel->logconfess($opts->{coldb}{error}="subprofile(): abstract method called");
 }
 
+##--------------------------------------------------------------
+## Relation API: default: subprofile2
+
 ## \%slice2prf = $rel->subprofile2(\%slice2prf,\%opts)
 ##  + populate f2 frequencies for profiles in \%slice2prf
 ##  + %opts: as for subprofile1()
@@ -316,8 +370,20 @@ sub subprofile2 {
   return $_[1];
 }
 
-##==============================================================================
-## Relation API: default: qinfo()
+##--------------------------------------------------------------
+## Relation API: default: subextend
+
+## \%slice2prf = $rel->subextend(\%slice2prf,\%opts)
+##  + populate f2 frequencies for profiles in \%slice2prf
+##  + %opts: as for subprofile1()
+##  + default implementation just pukes
+sub subextend {
+  my ($rel,$slice2prf,$opts) = @_;
+  $rel->logconfess($opts->{coldb}{error}="subextend() method not supported");
+}
+
+##--------------------------------------------------------------
+## Relation API: default: qinfo
 
 ## \%qinfo = $rel->qinfo($coldb, %opts)
 ##  + get query-info hash for profile administrivia (ddc hit links)
