@@ -64,14 +64,12 @@ sub new {
 		    pack_i=>'N',
 		    pack_f=>'N',
 		    pack_d=>'n',
-		    r1 => $PFCLASS->new(),
-		    r2 => $PFCLASS->new(),
-		    r3 => $PFCLASS->new(),
 		    N  => 0,
 		    version => $DiaColloDB::VERSION,
 		    logCompat => 'warn',
 		    @_
 		   }, (ref($that)||$that));
+  $cof->{$_} //= $cof->mmclass($PFCLASS)->new() foreach (qw(r1 r2 r3));
   $cof->{class} = ref($cof);
   return $cof->open() if (defined($cof->{base}));
   return $cof;
@@ -535,7 +533,10 @@ sub f12 {
 }
 
 ##==============================================================================
-## Relation API: default: profiling
+## Relation API: default
+
+##--------------------------------------------------------------
+## Relation API: default: profile
 
 ## \%slice2prf = $rel->subprofile1(\@tids,\%opts)
 ##  + get slice-wise joint co-frequency profile(s) for @tids (db must be opened; f1 and f12 only)
@@ -614,6 +615,9 @@ sub subprofile1 {
   return \%slice2prf;
 }
 
+##--------------------------------------------------------------
+## Relation API: default: subprofile2
+
 ##  \%slice2prf = $rel->subprofile2(\%slice2prf, \%opts)
 ##  + populate f2 frequencies for profiles in \%slice2prf
 ##  + %opts: as for subprofile1()
@@ -688,9 +692,20 @@ sub subprofile2 {
   return $slice2prf;
 }
 
+##--------------------------------------------------------------
+## Relation API: default: subextend
 
-##==============================================================================
-## Relation API: default: query info
+## \%slice2prf = $rel->subextend(\%slice2prf,\%opts)
+##  + populate f2 frequencies for profiles in \%slice2prf
+##  + %opts: as for subprofile1()
+##  + override calls subprofile2()
+sub subextend {
+  my $cof = shift;
+  return $cof->subprofile2(@_);
+}
+
+##--------------------------------------------------------------
+## Relation API: default: qinfo
 
 ## \%qinfo = $rel->qinfo($coldb, %opts)
 ##  + get query-info hash for profile administrivia (ddc hit links)
