@@ -429,7 +429,10 @@ sub dbinfo {
 
 
 ##==============================================================================
-## Relation API: default: profiling
+## Relation API: default
+
+##--------------------------------------------------------------
+## Relation API: default: profile
 
 ## \%slice2prf = $rel->subprofile1(\@tids,\%opts)
 ##  + get slice-wise joint co-frequency profile(s) for @tids (db must be opened; f1 and f12 only)
@@ -494,13 +497,27 @@ sub subprofile1 {
   return \%slice2prf;
 }
 
+##--------------------------------------------------------------
+## Relation API: default: subprofile2
+
 ##  \%slice2prf = $rel->subprofile2(\%slice2prf, \%opts)
 ##  + populate f2 frequencies for profiles in \%slice2prf
 ##  + %opts: as for subprofile1()
 ##  + INHERITED from DiaColloDB::Relation : no-op
 
-##==============================================================================
-## Relation API: default: query info
+##--------------------------------------------------------------
+## Relation API: default: subextend
+
+## \%slice2prf = $rel->subextend(\%slice2prf,\%opts)
+##  + populate f2 frequencies for profiles in \%slice2prf
+##  + %opts: as for subprofile1()
+##  + override returns empty meta-profile (no-op)
+sub subextend {
+  return DiaColloDB::Profile::Multi->new();
+}
+
+##--------------------------------------------------------------
+## Relation API: default: qinfo
 
 ## \%qinfo = $rel->qinfo($coldb, %opts)
 ##  + get query-info hash for profile administrivia (ddc hit links)
